@@ -15,24 +15,18 @@
 # specific language governing permissions and limitations
 # under the License.
 
-from typing import Any, Iterable, Literal, Tuple
+from typing import Any, Iterable, Literal, Tuple, Union
 
-from nanoarrow._lib import (
-    CArray,
-    CArrayBuilder,
-    CArrayView,
-    CArrowType,
-    CBuffer,
-    CBufferBuilder,
-    CSchema,
-    CSchemaBuilder,
-    NoneAwareWrapperIterator,
-    _obj_is_buffer,
-    _obj_is_capsule,
-)
+from nanoarrow._array import CArray, CArrayBuilder, CArrayView, CDeviceArray
+from nanoarrow._buffer import CBuffer, CBufferBuilder, NoneAwareWrapperIterator
+from nanoarrow._device import DEVICE_CPU, Device
+from nanoarrow._schema import CSchema, CSchemaBuilder
+from nanoarrow._utils import obj_is_buffer, obj_is_capsule
 from nanoarrow.c_buffer import c_buffer
 from nanoarrow.c_schema import c_schema, c_schema_view
 
+from nanoarrow import _types
+
 
 def c_array(obj, schema=None) -> CArray:
     """ArrowArray wrapper
@@ -97,7 +91,7 @@ def c_array(obj, schema=None) -> CArray:
         )
 
     # Try import of bare capsule
-    if _obj_is_capsule(obj, "arrow_array"):
+    if obj_is_capsule(obj, "arrow_array"):
         if schema is None:
             schema_capsule = CSchema.allocate()._capsule
         else:
@@ -138,7 +132,7 @@ def _resolve_builder(obj):
     if _obj_is_empty(obj):
         return EmptyArrayBuilder
 
-    if _obj_is_buffer(obj):
+    if obj_is_buffer(obj):
         return ArrayFromPyBufferBuilder
 
     if _obj_is_iterable(obj):
@@ -208,6 +202,7 @@ def c_array_from_buffers(
     children: Iterable[Any] = (),
     validation_level: Literal[None, "full", "default", "minimal", "none"] = None,
     move: bool = False,
+    device: Union[Device, None] = None,
 ) -> CArray:
     """Create an ArrowArray wrapper from components
 
@@ -248,6 +243,10 @@ def c_array_from_buffers(
     move : bool, optional
         Use ``True`` to move ownership of any input buffers or children to the
         output array.
+    device : Device, optional
+        An explicit device to use when constructing this array. If specified,
+        this function will construct a :class:`CDeviceArray`; if unspecified,
+        this function will construct a :class:`CArray` on the CPU device.
 
     Examples
     --------
@@ -265,8 +264,14 @@ def c_array_from_buffers(
     - dictionary: NULL
     - children[0]:
     """
+    if device is None:
+        explicit_device = False
+        device = DEVICE_CPU
+    else:
+        explicit_device = True
+
     schema = c_schema(schema)
-    builder = CArrayBuilder.allocate()
+    builder = CArrayBuilder.allocate(device)
 
     # Ensures that the output array->n_buffers is set and that the correct number
     # of children have been initialized.
@@ -289,7 +294,10 @@ def c_array_from_buffers(
     for child_src in children:
         # If we're setting a CArray from something else, we can avoid an extra
         # level of Python wrapping by using move=True
-        move = move or not isinstance(child_src, CArray)
+        move = move or not isinstance(child_src, (CArray, CDeviceArray))
+        if move and isinstance(child_src, CDeviceArray):
+            child_src = child_src.array
+
         builder.set_child(n_children, c_array(child_src), move=move)
         n_children += 1
 
@@ -304,8 +312,12 @@ def c_array_from_buffers(
     # Calculates the null count if -1 (and if applicable)
     builder.resolve_null_count()
 
-    # Validate + finish
-    return builder.finish(validation_level=validation_level)
+    # Validate + finish. If device is specified (even CPU), always
+    # return a device array.
+    if not explicit_device:
+        return builder.finish(validation_level=validation_level)
+    else:
+        return builder.finish_device()
 
 
 class ArrayBuilder:
@@ -357,7 +369,7 @@ class EmptyArrayBuilder(ArrayBuilder):
 
     @classmethod
     def infer_schema(cls, obj) -> Tuple[Any, CSchema]:
-        return obj, CSchemaBuilder.allocate().set_type(CArrowType.NA)
+        return obj, CSchemaBuilder.allocate().set_type(_types.NA)
 
     def start_building(self) -> None:
         self._c_builder.start_appending()
@@ -386,18 +398,16 @@ class ArrayFromPyBufferBuilder(ArrayBuilder):
         element_size_bits = obj.element_size_bits
 
         # Fixed-size binary needs a schema
-        if type_id == CArrowType.BINARY and element_size_bits != 0:
+        if type_id == _types.BINARY and element_size_bits != 0:
             schema = (
                 CSchemaBuilder.allocate()
-                .set_type_fixed_size(
-                    CArrowType.FIXED_SIZE_BINARY, element_size_bits // 8
-                )
+                .set_type_fixed_size(_types.FIXED_SIZE_BINARY, element_size_bits // 8)
                 .finish()
             )
-        elif type_id == CArrowType.STRING:
-            schema = CSchemaBuilder.allocate().set_type(CArrowType.INT8).finish()
-        elif type_id == CArrowType.BINARY:
-            schema = CSchemaBuilder.allocate().set_type(CArrowType.UINT8).finish()
+        elif type_id == _types.STRING:
+            schema = CSchemaBuilder.allocate().set_type(_types.INT8).finish()
+        elif type_id == _types.BINARY:
+            schema = CSchemaBuilder.allocate().set_type(_types.UINT8).finish()
         else:
             schema = CSchemaBuilder.allocate().set_type(type_id).finish()
 
@@ -406,7 +416,7 @@ class ArrayFromPyBufferBuilder(ArrayBuilder):
     def __init__(self, schema):
         super().__init__(schema)
 
-        if self._schema_view.buffer_format is None:
+        if self._schema_view.storage_buffer_format is None:
             raise ValueError(
                 f"Can't build array of type {self._schema_view.type} from PyBuffer"
             )
@@ -508,7 +518,7 @@ class ArrayFromIterableBuilder(ArrayBuilder):
     def _append_using_array(self, obj: Iterable) -> None:
         from array import array
 
-        py_array = array(self._schema_view.buffer_format, obj)
+        py_array = array(self._schema_view.storage_buffer_format, obj)
         buffer = CBuffer.from_pybuffer(py_array)
         self._c_builder.set_buffer(1, buffer, move=True)
         self._c_builder.set_length(len(buffer))
@@ -529,26 +539,32 @@ class ArrayFromIterableBuilder(ArrayBuilder):
 
 
 _ARRAY_BUILDER_FROM_ITERABLE_METHOD = {
-    CArrowType.BOOL: "_append_using_buffer_builder",
-    CArrowType.HALF_FLOAT: "_append_using_buffer_builder",
-    CArrowType.INTERVAL_MONTH_DAY_NANO: "_append_using_buffer_builder",
-    CArrowType.INTERVAL_DAY_TIME: "_append_using_buffer_builder",
-    CArrowType.INTERVAL_MONTHS: "_append_using_buffer_builder",
-    CArrowType.BINARY: "_append_bytes",
-    CArrowType.LARGE_BINARY: "_append_bytes",
-    CArrowType.FIXED_SIZE_BINARY: "_append_bytes",
-    CArrowType.STRING: "_append_strings",
-    CArrowType.LARGE_STRING: "_append_strings",
-    CArrowType.INT8: "_append_using_array",
-    CArrowType.UINT8: "_append_using_array",
-    CArrowType.INT16: "_append_using_array",
-    CArrowType.UINT16: "_append_using_array",
-    CArrowType.INT32: "_append_using_array",
-    CArrowType.UINT32: "_append_using_array",
-    CArrowType.INT64: "_append_using_array",
-    CArrowType.UINT64: "_append_using_array",
-    CArrowType.FLOAT: "_append_using_array",
-    CArrowType.DOUBLE: "_append_using_array",
+    _types.BOOL: "_append_using_buffer_builder",
+    _types.HALF_FLOAT: "_append_using_buffer_builder",
+    _types.INTERVAL_MONTH_DAY_NANO: "_append_using_buffer_builder",
+    _types.INTERVAL_DAY_TIME: "_append_using_buffer_builder",
+    _types.INTERVAL_MONTHS: "_append_using_buffer_builder",
+    _types.BINARY: "_append_bytes",
+    _types.LARGE_BINARY: "_append_bytes",
+    _types.FIXED_SIZE_BINARY: "_append_bytes",
+    _types.BINARY_VIEW: "_append_bytes",
+    _types.STRING: "_append_strings",
+    _types.LARGE_STRING: "_append_strings",
+    _types.STRING_VIEW: "_append_strings",
+    _types.INT8: "_append_using_array",
+    _types.UINT8: "_append_using_array",
+    _types.INT16: "_append_using_array",
+    _types.UINT16: "_append_using_array",
+    _types.INT32: "_append_using_array",
+    _types.UINT32: "_append_using_array",
+    _types.INT64: "_append_using_array",
+    _types.UINT64: "_append_using_array",
+    _types.FLOAT: "_append_using_array",
+    _types.DOUBLE: "_append_using_array",
+    _types.TIMESTAMP: "_append_using_array",
+    _types.DATE32: "_append_using_array",
+    _types.DATE64: "_append_using_array",
+    _types.DURATION: "_append_using_array",
 }
 
 _ARRAY_BUILDER_FROM_NULLABLE_ITERABLE_METHOD = {
